@@ -58,6 +58,7 @@
 
 #include "nsDOMThreadService.h"
 #include "nsDOMWorkerEvents.h"
+#include "nsDOMWorkerLocation.h"
 #include "nsDOMWorkerNavigator.h"
 #include "nsDOMWorkerPool.h"
 #include "nsDOMWorkerScriptLoader.h"
@@ -139,6 +140,12 @@ nsDOMWorkerFunctions::MakeTimeout(JSContext* aCx,
 
   PRUint32 id = worker->NextTimeoutId();
 
+  if (worker->IsClosing()) {
+    // Timeouts won't run in the close handler, fake success and bail.
+    *aRval = INT_TO_JSVAL(id);
+    return JS_TRUE;
+  }
+
   nsRefPtr<nsDOMWorkerTimeout> timeout = new nsDOMWorkerTimeout(worker, id);
   if (!timeout) {
     JS_ReportOutOfMemory(aCx);
@@ -343,15 +350,9 @@ nsDOMWorkerFunctions::NewWorker(JSContext* aCx,
     return JS_FALSE;
   }
 
-  nsRefPtr<nsDOMWorkerPool> pool = worker->Pool();
-  if (!pool) {
-    JS_ReportError(aCx, "Couldn't get pool from worker!");
-    return JS_FALSE;
-  }
-
   // This pointer is protected by our pool, but it is *not* threadsafe and must
   // not be used in any way other than to pass it along to the Initialize call.
-  nsIScriptGlobalObject* owner = pool->ScriptGlobalObject();
+  nsIScriptGlobalObject* owner = worker->Pool()->ScriptGlobalObject();
   if (!owner) {
     JS_ReportError(aCx, "Couldn't get owner from pool!");
     return JS_FALSE;
@@ -427,66 +428,46 @@ WriteCallback(const jschar* aBuffer,
 }
 
 static nsresult
-GetStringForArgument(nsAString& aString,
+GetStringForArgument(JSContext* aCx,
+                     jsval aVal,
                      PRBool* aIsJSON,
-                     PRBool* aIsPrimitive)
+                     PRBool* aIsPrimitive,
+                     nsAutoJSValHolder& _retval)
 {
   NS_ASSERTION(aIsJSON && aIsPrimitive, "Null pointer!");
 
-  nsIXPConnect* xpc = nsContentUtils::XPConnect();
-  NS_ENSURE_TRUE(xpc, NS_ERROR_UNEXPECTED);
+  if (JSVAL_IS_STRING(aVal)) {
+    if (!JS_MakeStringImmutable(aCx, JSVAL_TO_STRING(aVal))) {
+      return NS_ERROR_FAILURE;
+    }
 
-  nsAXPCNativeCallContext* cc;
-  nsresult rv = xpc->GetCurrentNativeCallContext(&cc);
-  NS_ENSURE_SUCCESS(rv, rv);
-  NS_ENSURE_TRUE(cc, NS_ERROR_UNEXPECTED);
-
-  PRUint32 argc;
-  rv = cc->GetArgc(&argc);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  if (!argc) {
-    return NS_ERROR_XPC_NOT_ENOUGH_ARGS;
-  }
-
-  jsval* argv;
-  rv = cc->GetArgvPtr(&argv);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  JSContext* cx;
-  rv = cc->GetJSContext(&cx);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  JSAutoRequest ar(cx);
-
-  if (JSVAL_IS_STRING(argv[0])) {
-    aString.Assign(nsDependentJSString(JSVAL_TO_STRING(argv[0])));
     *aIsJSON = *aIsPrimitive = PR_FALSE;
+    _retval = aVal;
     return NS_OK;
   }
 
   nsAutoJSValHolder jsonVal;
 
-  JSBool ok = jsonVal.Hold(cx);
+  JSBool ok = jsonVal.Hold(aCx);
   NS_ENSURE_TRUE(ok, NS_ERROR_FAILURE);
 
-  if (JSVAL_IS_PRIMITIVE(argv[0])) {
+  if (JSVAL_IS_PRIMITIVE(aVal)) {
     // Only objects can be serialized through JSON, currently, so if we've been
     // given a primitive we set it as a property on a dummy object before
     // sending it to the serializer.
-    JSObject* obj = JS_NewObject(cx, NULL, NULL, NULL);
+    JSObject* obj = JS_NewObject(aCx, NULL, NULL, NULL);
     NS_ENSURE_TRUE(obj, NS_ERROR_OUT_OF_MEMORY);
 
     jsonVal = obj;
 
-    ok = JS_DefineProperty(cx, obj, JSON_PRIMITIVE_PROPNAME, argv[0], NULL,
+    ok = JS_DefineProperty(aCx, obj, JSON_PRIMITIVE_PROPNAME, aVal, NULL,
                            NULL, JSPROP_ENUMERATE);
     NS_ENSURE_TRUE(ok, NS_ERROR_UNEXPECTED);
 
     *aIsPrimitive = PR_TRUE;
   }
   else {
-    jsonVal = argv[0];
+    jsonVal = aVal;
 
     *aIsPrimitive = PR_FALSE;
   }
@@ -495,9 +476,9 @@ GetStringForArgument(nsAString& aString,
   jsval* vp = jsonVal.ToJSValPtr();
 
   // This may change vp if there is a 'toJSON' function on the object.
-  ok = JS_TryJSON(cx, vp);
+  ok = JS_TryJSON(aCx, vp);
   if (!(ok && !JSVAL_IS_PRIMITIVE(*vp) &&
-        (type = JS_TypeOfValue(cx, *vp)) != JSTYPE_FUNCTION &&
+        (type = JS_TypeOfValue(aCx, *vp)) != JSTYPE_FUNCTION &&
         type != JSTYPE_XML)) {
     return NS_ERROR_INVALID_ARG;
   }
@@ -507,7 +488,8 @@ GetStringForArgument(nsAString& aString,
 
   nsJSONWriter writer;
 
-  ok = JS_Stringify(cx, jsonVal.ToJSValPtr(), NULL, JSVAL_NULL, WriteCallback, &writer);
+  ok = JS_Stringify(aCx, jsonVal.ToJSValPtr(), NULL, JSVAL_NULL, WriteCallback,
+                    &writer);
   if (!ok) {
     return NS_ERROR_XPC_BAD_CONVERT_JS;
   }
@@ -516,24 +498,35 @@ GetStringForArgument(nsAString& aString,
 
   writer.FlushBuffer();
 
-  aString.Assign(writer.mOutputString);
-  *aIsJSON = PR_TRUE;
+  _retval = nsDOMThreadService::ShareStringAsJSVal(aCx, writer.mOutputString);
+  if (!JSVAL_IS_STRING(_retval)) {
+    // Yuck, we can't share.
+    const jschar* buf =
+      reinterpret_cast<const jschar*>(writer.mOutputString.get());
+    JSString* str = JS_NewUCStringCopyN(aCx, buf, writer.mOutputString.Length());
+    if (!str) {
+      JS_ReportOutOfMemory(aCx);
+      return NS_ERROR_OUT_OF_MEMORY;
+    }
+    _retval = STRING_TO_JSVAL(str);
+  }
 
+  *aIsJSON = PR_TRUE;
   return NS_OK;
 }
 
 nsDOMWorkerScope::nsDOMWorkerScope(nsDOMWorker* aWorker)
 : mWorker(aWorker),
+  mWrappedNative(nsnull),
   mHasOnerror(PR_FALSE)
 {
   NS_ASSERTION(aWorker, "Null pointer!");
 }
 
-NS_IMPL_THREADSAFE_ISUPPORTS5(nsDOMWorkerScope, nsIWorkerScope,
-                                                nsIWorkerGlobalScope,
-                                                nsIDOMEventTarget,
-                                                nsIXPCScriptable,
-                                                nsIClassInfo)
+NS_IMPL_ISUPPORTS_INHERITED3(nsDOMWorkerScope, nsDOMWorkerMessageHandler,
+                                               nsIWorkerScope,
+                                               nsIWorkerGlobalScope,
+                                               nsIXPCScriptable)
 
 NS_IMPL_CI_INTERFACE_GETTER4(nsDOMWorkerScope, nsIWorkerScope,
                                                nsIWorkerGlobalScope,
@@ -563,6 +556,9 @@ nsDOMWorkerScope::GetHelperForLanguage(PRUint32 aLanguage,
 
 #define XPC_MAP_CLASSNAME nsDOMWorkerScope
 #define XPC_MAP_QUOTED_CLASSNAME "DedicatedWorkerGlobalScope"
+#define XPC_MAP_WANT_POSTCREATE
+#define XPC_MAP_WANT_TRACE
+#define XPC_MAP_WANT_FINALIZE
 
 #define XPC_MAP_FLAGS                                      \
   nsIXPCScriptable::USE_JSSTUB_FOR_ADDPROPERTY           | \
@@ -577,6 +573,45 @@ nsDOMWorkerScope::GetHelperForLanguage(PRUint32 aLanguage,
 
 #include "xpc_map_end.h"
 
+NS_IMETHODIMP
+nsDOMWorkerScope::PostCreate(nsIXPConnectWrappedNative*  aWrapper,
+                             JSContext* /* aCx */,
+                             JSObject* /* aObj */)
+{
+  NS_ASSERTION(!mWrappedNative, "Already got a wrapper?!");
+  mWrappedNative = aWrapper;
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsDOMWorkerScope::Trace(nsIXPConnectWrappedNative* /* aWrapper */,
+                        JSTracer* aTracer,
+                        JSObject* /*aObj */)
+{
+  NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
+  nsDOMWorkerMessageHandler::Trace(aTracer);
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsDOMWorkerScope::Finalize(nsIXPConnectWrappedNative* /* aWrapper */,
+                           JSContext* /* aCx */,
+                           JSObject* /* aObj */)
+{
+  NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
+  ClearAllListeners();
+  mWrappedNative = nsnull;
+  return NS_OK;
+}
+
+already_AddRefed<nsIXPConnectWrappedNative>
+nsDOMWorkerScope::GetWrappedNative()
+{
+  nsCOMPtr<nsIXPConnectWrappedNative> wrappedNative = mWrappedNative;
+  NS_ASSERTION(wrappedNative, "Null wrapped native!");
+  return wrappedNative.forget();
+}
+
 NS_IMETHODIMP
 nsDOMWorkerScope::AddProperty(nsIXPConnectWrappedNative* aWrapper,
                               JSContext* aCx,
@@ -656,6 +691,18 @@ nsDOMWorkerScope::GetNavigator(nsIWorkerNavigator** _retval)
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsDOMWorkerScope::GetLocation(nsIWorkerLocation** _retval)
+{
+  NS_ASSERTION(!NS_IsMainThread(), "Wrong thread!");
+
+  nsCOMPtr<nsIWorkerLocation> location = mWorker->GetLocation();
+  NS_ASSERTION(location, "This should never be null!");
+
+  location.forget(_retval);
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsDOMWorkerScope::GetOnerror(nsIDOMEventListener** aOnerror)
 {
@@ -685,7 +732,7 @@ nsDOMWorkerScope::GetOnerror(nsIDOMEventListener** aOnerror)
   }
 
   nsCOMPtr<nsIDOMEventListener> listener =
-    mWorker->mInnerHandler->GetOnXListener(NS_LITERAL_STRING("error"));
+    GetOnXListener(NS_LITERAL_STRING("error"));
   listener.forget(aOnerror);
 
   return NS_OK;
@@ -702,8 +749,7 @@ nsDOMWorkerScope::SetOnerror(nsIDOMEventListener* aOnerror)
 
   mHasOnerror = PR_TRUE;
 
-  return mWorker->mInnerHandler->SetOnXListener(NS_LITERAL_STRING("error"),
-                                                aOnerror);
+  return SetOnXListener(NS_LITERAL_STRING("error"), aOnerror);
 }
 
 NS_IMETHODIMP
@@ -715,13 +761,15 @@ nsDOMWorkerScope::PostMessage(/* JSObject aMessage */)
     return NS_ERROR_ABORT;
   }
 
-  nsString message;
-  PRBool isJSON, isPrimitive;
+  return mWorker->PostMessageInternal(PR_FALSE);
+}
 
-  nsresult rv = GetStringForArgument(message, &isJSON, &isPrimitive);
-  NS_ENSURE_SUCCESS(rv, rv);
+NS_IMETHODIMP
+nsDOMWorkerScope::Close()
+{
+  NS_ASSERTION(!NS_IsMainThread(), "Wrong thread!");
 
-  return mWorker->PostMessageInternal(message, isJSON, isPrimitive, PR_FALSE);
+  return mWorker->Close();
 }
 
 NS_IMETHODIMP
@@ -735,7 +783,7 @@ nsDOMWorkerScope::GetOnmessage(nsIDOMEventListener** aOnmessage)
   }
 
   nsCOMPtr<nsIDOMEventListener> listener =
-    mWorker->mInnerHandler->GetOnXListener(NS_LITERAL_STRING("message"));
+    GetOnXListener(NS_LITERAL_STRING("message"));
   listener.forget(aOnmessage);
 
   return NS_OK;
@@ -750,8 +798,39 @@ nsDOMWorkerScope::SetOnmessage(nsIDOMEventListener* aOnmessage)
     return NS_ERROR_ABORT;
   }
 
-  return mWorker->mInnerHandler->SetOnXListener(NS_LITERAL_STRING("message"),
-                                                aOnmessage);
+  return SetOnXListener(NS_LITERAL_STRING("message"), aOnmessage);
+}
+
+NS_IMETHODIMP
+nsDOMWorkerScope::GetOnclose(nsIDOMEventListener** aOnclose)
+{
+  NS_ASSERTION(!NS_IsMainThread(), "Wrong thread!");
+  NS_ENSURE_ARG_POINTER(aOnclose);
+
+  if (mWorker->IsCanceled()) {
+    return NS_ERROR_ABORT;
+  }
+
+  nsCOMPtr<nsIDOMEventListener> listener =
+    GetOnXListener(NS_LITERAL_STRING("close"));
+  listener.forget(aOnclose);
+
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsDOMWorkerScope::SetOnclose(nsIDOMEventListener* aOnclose)
+{
+  NS_ASSERTION(!NS_IsMainThread(), "Wrong thread!");
+
+  if (mWorker->IsCanceled()) {
+    return NS_ERROR_ABORT;
+  }
+
+  nsresult rv = SetOnXListener(NS_LITERAL_STRING("close"), aOnclose);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -765,8 +844,8 @@ nsDOMWorkerScope::AddEventListener(const nsAString& aType,
     return NS_ERROR_ABORT;
   }
 
-  return mWorker->mInnerHandler->AddEventListener(aType, aListener,
-                                                  aUseCapture);
+  return nsDOMWorkerMessageHandler::AddEventListener(aType, aListener,
+                                                     aUseCapture);
 }
 
 NS_IMETHODIMP
@@ -780,8 +859,8 @@ nsDOMWorkerScope::RemoveEventListener(const nsAString& aType,
     return NS_ERROR_ABORT;
   }
 
-  return mWorker->mInnerHandler->RemoveEventListener(aType, aListener,
-                                                     aUseCapture);
+  return nsDOMWorkerMessageHandler::RemoveEventListener(aType, aListener,
+                                                        aUseCapture);
 }
 
 NS_IMETHODIMP
@@ -794,7 +873,7 @@ nsDOMWorkerScope::DispatchEvent(nsIDOMEvent* aEvent,
     return NS_ERROR_ABORT;
   }
 
-  return mWorker->mInnerHandler->DispatchEvent(aEvent, _retval);
+  return nsDOMWorkerMessageHandler::DispatchEvent(aEvent, _retval);
 }
 
 class nsWorkerHoldingRunnable : public nsIRunnable
@@ -809,6 +888,10 @@ public:
     return NS_OK;
   }
 
+  void ReplaceWrappedNative(nsIXPConnectWrappedNative* aWrappedNative) {
+    mWorkerWN = aWrappedNative;
+  }
+
 protected:
   virtual ~nsWorkerHoldingRunnable() { }
 
@@ -852,9 +935,20 @@ public:
       return NS_ERROR_ABORT;
     }
 
+    // If the worker is suspended and we're running on the main thread then we
+    // can't actually dispatch the event yet. Instead we queue it for whenever
+    // we resume.
+    if (mWorker->IsSuspended() && NS_IsMainThread()) {
+      if (!mWorker->QueueSuspendedRunnable(this)) {
+        NS_ERROR("Out of memory?!");
+        return NS_ERROR_ABORT;
+      }
+      return NS_OK;
+    }
+
     nsCOMPtr<nsIDOMEventTarget> target = mToInner ?
-      static_cast<nsIDOMEventTarget*>(mWorker->GetInnerScope()) :
-      static_cast<nsIDOMEventTarget*>(mWorker);
+      static_cast<nsDOMWorkerMessageHandler*>(mWorker->GetInnerScope()) :
+      static_cast<nsDOMWorkerMessageHandler*>(mWorker);
 
     NS_ASSERTION(target, "Null target!");
     NS_ENSURE_TRUE(target, NS_ERROR_FAILURE);
@@ -870,24 +964,6 @@ protected:
 
 NS_IMPL_ISUPPORTS_INHERITED0(nsDOMFireEventRunnable, nsWorkerHoldingRunnable)
 
-class nsCancelDOMWorkerRunnable : public nsWorkerHoldingRunnable
-{
-  NS_DECL_ISUPPORTS_INHERITED
-
-  nsCancelDOMWorkerRunnable(nsDOMWorker* aWorker)
-  : nsWorkerHoldingRunnable(aWorker) { }
-
-  NS_IMETHOD Run() {
-    NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
-    if (!mWorker->IsCanceled()) {
-      mWorker->Cancel();
-    }
-    return NS_OK;
-  }
-};
-
-NS_IMPL_ISUPPORTS_INHERITED0(nsCancelDOMWorkerRunnable, nsWorkerHoldingRunnable)
-
 // Standard NS_IMPL_THREADSAFE_ADDREF without the logging stuff (since this
 // class is made to be inherited anyway).
 NS_IMETHODIMP_(nsrefcnt)
@@ -958,10 +1034,10 @@ nsDOMWorker::nsDOMWorker(nsDOMWorker* aParent,
   mFeatureSuspendDepth(0),
   mWrappedNative(nsnull),
   mErrorHandlerRecursionCount(0),
-  mCanceled(PR_FALSE),
+  mStatus(eRunning),
+  mExpirationTime(0),
   mSuspended(PR_FALSE),
-  mCompileAttempted(PR_FALSE),
-  mTerminated(PR_FALSE)
+  mCompileAttempted(PR_FALSE)
 {
 #ifdef DEBUG
   PRBool mainThread = NS_IsMainThread();
@@ -980,6 +1056,7 @@ nsDOMWorker::~nsDOMWorker()
   }
 
   NS_ASSERTION(!mFeatures.Length(), "Live features!");
+  NS_ASSERTION(!mQueuedRunnables.Length(), "Events that never ran!");
 
   nsCOMPtr<nsIThread> mainThread;
   NS_GetMainThread(getter_AddRefs(mainThread));
@@ -1010,16 +1087,17 @@ nsDOMWorker::NewWorker(nsISupports** aNewObject)
   return NS_OK;
 }
 
-NS_IMPL_THREADSAFE_ADDREF(nsDOMWorker)
-NS_IMPL_THREADSAFE_RELEASE(nsDOMWorker)
+NS_IMPL_ADDREF_INHERITED(nsDOMWorker, nsDOMWorkerMessageHandler)
+NS_IMPL_RELEASE_INHERITED(nsDOMWorker, nsDOMWorkerMessageHandler)
 
 NS_INTERFACE_MAP_BEGIN(nsDOMWorker)
   NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIWorker)
   NS_INTERFACE_MAP_ENTRY(nsIWorker)
   NS_INTERFACE_MAP_ENTRY(nsIAbstractWorker)
-  NS_INTERFACE_MAP_ENTRY(nsIDOMEventTarget)
+  NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsIDOMEventTarget, nsDOMWorkerMessageHandler)
   NS_INTERFACE_MAP_ENTRY(nsIXPCScriptable)
   NS_INTERFACE_MAP_ENTRY(nsIJSNativeInitializer)
+  NS_INTERFACE_MAP_ENTRY(nsITimerCallback)
   if (aIID.Equals(NS_GET_IID(nsIClassInfo))) {
     foundInterface = static_cast<nsIClassInfo*>(&sDOMWorkerClassInfo);
   } else
@@ -1049,6 +1127,7 @@ nsDOMWorker::PostCreate(nsIXPConnectWrappedNative* aWrapper,
                         JSContext* /* aCx */,
                         JSObject* /* aObj */)
 {
+  nsAutoLock lock(mLock);
   mWrappedNative = aWrapper;
   return NS_OK;
 }
@@ -1060,16 +1139,14 @@ nsDOMWorker::Trace(nsIXPConnectWrappedNative* /* aWrapper */,
 {
   NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
 
-  if (!IsCanceled()) {
-    if (mGlobal) {
-      JS_SET_TRACING_DETAILS(aTracer, nsnull, this, 0);
-      JS_CallTracer(aTracer, mGlobal, JSTRACE_OBJECT);
-    }
-    // We should never get null handlers here if our call to Initialize succeeded.
-    NS_ASSERTION(mInnerHandler && mOuterHandler, "Shouldn't be possible!");
+  PRBool canceled = PR_FALSE;
+  {
+    nsAutoLock lock(mLock);
+    canceled = mStatus == eKilled;
+  }
 
-    mInnerHandler->Trace(aTracer);
-    mOuterHandler->Trace(aTracer);
+  if (!canceled) {
+    nsDOMWorkerMessageHandler::Trace(aTracer);
   }
 
   return NS_OK;
@@ -1083,19 +1160,17 @@ nsDOMWorker::Finalize(nsIXPConnectWrappedNative* /* aWrapper */,
   NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
 
   // Don't leave dangling JSObject pointers in our handlers!
-  mInnerHandler->ClearAllListeners();
-  mOuterHandler->ClearAllListeners();
+  ClearAllListeners();
 
   // Clear our wrapped native now that it has died.
-  mWrappedNative = nsnull;
+  {
+    nsAutoLock lock(mLock);
+    mWrappedNative = nsnull;
+  }
 
-  // We no longer need to keep our inner scope.
-  mGlobal = NULL;
-  mInnerScope = nsnull;
-
-  // And we can let our parent die now too.
-  mParent = nsnull;
-  mParentWN = nsnull;
+  // Do this *after* we null out mWrappedNative so that we don't hand out a
+  // freed pointer.
+  TerminateInternal(PR_TRUE);
 
   return NS_OK;
 }
@@ -1135,12 +1210,6 @@ nsDOMWorker::InitializeInternal(nsIScriptGlobalObject* aOwner,
   mLock = nsAutoLock::NewLock("nsDOMWorker::mLock");
   NS_ENSURE_TRUE(mLock, NS_ERROR_OUT_OF_MEMORY);
 
-  mInnerHandler = new nsDOMWorkerMessageHandler();
-  NS_ENSURE_TRUE(mInnerHandler, NS_ERROR_OUT_OF_MEMORY);
-
-  mOuterHandler = new nsDOMWorkerMessageHandler();
-  NS_ENSURE_TRUE(mOuterHandler, NS_ERROR_OUT_OF_MEMORY);
-
   NS_ASSERTION(!mGlobal, "Already got a global?!");
 
   nsIXPConnect* xpc = nsContentUtils::XPConnect();
@@ -1153,6 +1222,16 @@ nsDOMWorker::InitializeInternal(nsIScriptGlobalObject* aOwner,
 
   NS_ASSERTION(mWrappedNative, "Post-create hook should have set this!");
 
+  mKillTimer = do_CreateInstance(NS_TIMER_CONTRACTID, &rv);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsCOMPtr<nsIThread> mainThread;
+  rv = NS_GetMainThread(getter_AddRefs(mainThread));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  rv = mKillTimer->SetTarget(mainThread);
+  NS_ENSURE_SUCCESS(rv, rv);
+
   // This is pretty cool - all we have to do to get our script executed is to
   // pass a no-op runnable to the thread service and it will make sure we have
   // a context and global object.
@@ -1177,47 +1256,272 @@ nsDOMWorker::InitializeInternal(nsIScriptGlobalObject* aOwner,
 void
 nsDOMWorker::Cancel()
 {
+  // Called by the pool when the window that created us is being torn down. Must
+  // always be on the main thread.
   NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
-  mCanceled = PR_TRUE;
 
-  CancelFeatures();
+  // We set the eCanceled status to indicate this. It behaves just like the
+  // eTerminated status (canceled while close runnable is unscheduled, not
+  // canceled while close runnable is running) except that it always reports
+  // that it is canceled when running on the main thread. This status trumps all
+  // others (except eKilled). Have to do this because the window that created
+  // us has gone away and had its scope cleared so XPConnect will assert all
+  // over the place if we try to run anything.
+
+  PRBool enforceTimeout = PR_FALSE;
+  {
+    nsAutoLock lock(mLock);
+
+    NS_ASSERTION(mStatus != eCanceled, "Canceled more than once?!");
+
+    if (mStatus == eKilled) {
+      return;
+    }
+
+    DOMWorkerStatus oldStatus = mStatus;
+    mStatus = eCanceled;
+    if (oldStatus != eRunning) {
+      enforceTimeout = PR_TRUE;
+    }
+  }
+
+  PRUint32 timeoutMS = nsDOMThreadService::GetWorkerCloseHandlerTimeoutMS();
+  NS_ASSERTION(timeoutMS, "This must not be 0!");
+
+#ifdef DEBUG
+  nsresult rv;
+#endif
+  if (enforceTimeout) {
+    // Tell the thread service to enforce a timeout on the close handler that
+    // is already scheduled.
+    nsDOMThreadService::get()->
+      SetWorkerTimeout(this, PR_MillisecondsToInterval(timeoutMS));
+
+#ifdef DEBUG
+    rv =
+#endif
+    mKillTimer->InitWithCallback(this, timeoutMS, nsITimer::TYPE_ONE_SHOT);
+    NS_WARN_IF_FALSE(NS_SUCCEEDED(rv), "Failed to init kill timer!");
+
+    return;
+  }
+
+#ifdef DEBUG
+  rv =
+#endif
+  FireCloseRunnable(PR_MillisecondsToInterval(timeoutMS), PR_TRUE, PR_FALSE);
+  NS_WARN_IF_FALSE(NS_SUCCEEDED(rv), "Failed to fire close runnable!");
+}
+
+void
+nsDOMWorker::Kill()
+{
+  // Cancel all features and set our status to eKilled. This should only be
+  // called on the main thread by the thread service or our kill timer to
+  // indicate that the worker's close handler has run (or timed out).
+  NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
+  NS_ASSERTION(IsClosing(), "Close handler should have run by now!");
+
+  // If the close handler finished before our kill timer then we don't need it
+  // any longer.
+  if (mKillTimer) {
+    mKillTimer->Cancel();
+    mKillTimer = nsnull;
+  }
+
+  PRUint32 count, index;
+  nsAutoTArray<nsRefPtr<nsDOMWorkerFeature>, 20> features;
+  {
+    nsAutoLock lock(mLock);
+
+    if (mStatus == eKilled) {
+      NS_ASSERTION(mFeatures.Length() == 0, "Features added after killed!");
+      return;
+    }
+    mStatus = eKilled;
+
+    count = mFeatures.Length();
+    for (index = 0; index < count; index++) {
+      nsDOMWorkerFeature*& feature = mFeatures[index];
+
+#ifdef DEBUG
+      nsRefPtr<nsDOMWorkerFeature>* newFeature =
+#endif
+      features.AppendElement(feature);
+      NS_ASSERTION(newFeature, "Out of memory!");
+
+      feature->FreeToDie(PR_TRUE);
+    }
+
+    mFeatures.Clear();
+  }
+
+  count = features.Length();
+  for (index = 0; index < count; index++) {
+    features[index]->Cancel();
+  }
+
+  // Make sure we kill any queued runnables that we never had a chance to run.
+  mQueuedRunnables.Clear();
+
+  // We no longer need to keep our inner scope.
+  mInnerScope = nsnull;
+  mScopeWN = nsnull;
+  mGlobal = NULL;
+
+  // And we can let our parent die now too.
+  mParent = nsnull;
+  mParentWN = nsnull;
 }
 
 void
 nsDOMWorker::Suspend()
 {
   NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
-  NS_ASSERTION(!mSuspended, "Suspended more than once!");
-  mSuspended = PR_TRUE;
 
-  SuspendFeatures();
+  PRBool shouldSuspendFeatures;
+  {
+    nsAutoLock lock(mLock);
+    NS_ASSERTION(!mSuspended, "Suspended more than once!");
+    shouldSuspendFeatures = !mSuspended;
+    mSuspended = PR_TRUE;
+  }
+
+  if (shouldSuspendFeatures) {
+    SuspendFeatures();
+  }
 }
 
 void
 nsDOMWorker::Resume()
 {
   NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
-  NS_ASSERTION(mSuspended, "Not suspended!");
-  mSuspended = PR_FALSE;
 
-  ResumeFeatures();
+  PRBool shouldResumeFeatures;
+  {
+    nsAutoLock lock(mLock);
+#ifdef DEBUG
+    // Should only have a mismatch if GC or Cancel happened while suspended.
+    if (!mSuspended) {
+      NS_ASSERTION(mStatus == eCanceled ||
+                   (mStatus == eTerminated && !mWrappedNative),
+                   "Not suspended!");
+    }
+#endif
+    shouldResumeFeatures = mSuspended;
+    mSuspended = PR_FALSE;
+  }
+
+  if (shouldResumeFeatures) {
+    ResumeFeatures();
+  }
+
+  // Repost any events that were queued for the main thread while suspended.
+  PRUint32 count = mQueuedRunnables.Length();
+  for (PRUint32 index = 0; index < count; index++) {
+    NS_DispatchToCurrentThread(mQueuedRunnables[index]);
+  }
+  mQueuedRunnables.Clear();
+}
+
+PRBool
+nsDOMWorker::IsCanceled()
+{
+  nsAutoLock lock(mLock);
+  return IsCanceledNoLock();
+}
+
+PRBool
+nsDOMWorker::IsCanceledNoLock()
+{
+  // If we haven't started the close process then we're not canceled.
+  if (mStatus == eRunning) {
+    return PR_FALSE;
+  }
+
+  // There are several conditions under which we want JS code to abort and all
+  // other functions to bail:
+  // 1. If we've already run our close handler then we are canceled forevermore.
+  // 2. If we've been terminated then we want to pretend to be canceled until
+  //    our close handler is scheduled and running.
+  // 3. If we've been canceled then we pretend to be canceled until the close
+  //    handler has been scheduled.
+  // 4. If the close handler has run for longer than the allotted time then we
+  //    should be canceled as well.
+  // 5. If we're on the main thread then we'll pretend to be canceled if the
+  //    user has navigated away from the page.
+  return mStatus == eKilled ||
+         (mStatus == eTerminated && !mExpirationTime) ||
+         (mStatus == eCanceled && !mExpirationTime) ||
+         (mExpirationTime && mExpirationTime != PR_INTERVAL_NO_TIMEOUT &&
+          mExpirationTime <= PR_IntervalNow()) ||
+         (mStatus == eCanceled && NS_IsMainThread());
+}
+
+PRBool
+nsDOMWorker::IsClosing()
+{
+  nsAutoLock lock(mLock);
+  return mStatus != eRunning;
+}
+
+PRBool
+nsDOMWorker::IsSuspended()
+{
+  nsAutoLock lock(mLock);
+  return mSuspended;
 }
 
 nsresult
-nsDOMWorker::PostMessageInternal(const nsAString& aMessage,
-                                 PRBool aIsJSON,
-                                 PRBool aIsPrimitive,
-                                 PRBool aToInner)
+nsDOMWorker::PostMessageInternal(PRBool aToInner)
 {
+  nsIXPConnect* xpc = nsContentUtils::XPConnect();
+  NS_ENSURE_TRUE(xpc, NS_ERROR_UNEXPECTED);
+
+  nsAXPCNativeCallContext* cc;
+  nsresult rv = xpc->GetCurrentNativeCallContext(&cc);
+  NS_ENSURE_SUCCESS(rv, rv);
+  NS_ENSURE_TRUE(cc, NS_ERROR_UNEXPECTED);
+
+  PRUint32 argc;
+  rv = cc->GetArgc(&argc);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  if (!argc) {
+    return NS_ERROR_XPC_NOT_ENOUGH_ARGS;
+  }
+
+  jsval* argv;
+  rv = cc->GetArgvPtr(&argv);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  JSContext* cx;
+  rv = cc->GetJSContext(&cx);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  JSAutoRequest ar(cx);
+
+  nsAutoJSValHolder val;
+  if (!val.Hold(cx)) {
+    return NS_ERROR_FAILURE;
+  }
+
+  PRBool isJSON, isPrimitive;
+  rv = GetStringForArgument(cx, argv[0], &isJSON, &isPrimitive, val);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  NS_ASSERTION(JSVAL_IS_STRING(val), "Bad jsval!");
+
   nsRefPtr<nsDOMWorkerMessageEvent> message = new nsDOMWorkerMessageEvent();
   NS_ENSURE_TRUE(message, NS_ERROR_OUT_OF_MEMORY);
 
-  nsresult rv = message->InitMessageEvent(NS_LITERAL_STRING("message"),
-                                          PR_FALSE, PR_FALSE, aMessage,
-                                          EmptyString(), nsnull);
+  rv = message->InitMessageEvent(NS_LITERAL_STRING("message"), PR_FALSE,
+                                 PR_FALSE, EmptyString(), EmptyString(),
+                                 nsnull);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  message->SetJSONData(aIsJSON, aIsPrimitive);
+  rv = message->SetJSONData(cx, val, isJSON, isPrimitive);
+  NS_ENSURE_SUCCESS(rv, rv);
 
   nsRefPtr<nsDOMFireEventRunnable> runnable =
     new nsDOMFireEventRunnable(this, message, aToInner);
@@ -1254,6 +1558,8 @@ nsDOMWorker::SetGlobalForContext(JSContext* aCx)
     return PR_FALSE;
   }
 
+  JSAutoRequest ar(aCx);
+
   JS_SetGlobalObject(aCx, mGlobal);
   return PR_TRUE;
 }
@@ -1316,10 +1622,14 @@ nsDOMWorker::CompileGlobalObject(JSContext* aCx)
   PRBool success = JS_DefineFunctions(aCx, global, gDOMWorkerFunctions);
   NS_ENSURE_TRUE(success, PR_FALSE);
 
-  // From here on out we have to remember to null mGlobal and mInnerScope if
-  // something fails!
+  // From here on out we have to remember to null mGlobal, mInnerScope, and
+  // mScopeWN if something fails! We really don't need to hang on to mGlobal
+  // as long as we have mScopeWN, but it saves us a virtual call every time the
+  // worker is scheduled. Meh.
   mGlobal = global;
   mInnerScope = scope;
+  mScopeWN = scope->GetWrappedNative();
+  NS_ASSERTION(mScopeWN, "Should have a wrapped native here!");
 
   nsRefPtr<nsDOMWorkerScriptLoader> loader =
     new nsDOMWorkerScriptLoader(this);
@@ -1327,6 +1637,7 @@ nsDOMWorker::CompileGlobalObject(JSContext* aCx)
   if (!loader) {
     mGlobal = NULL;
     mInnerScope = nsnull;
+    mScopeWN = nsnull;
     return PR_FALSE;
   }
 
@@ -1334,6 +1645,7 @@ nsDOMWorker::CompileGlobalObject(JSContext* aCx)
   if (NS_FAILED(rv)) {
     mGlobal = NULL;
     mInnerScope = nsnull;
+    mScopeWN = nsnull;
     return PR_FALSE;
   }
 
@@ -1344,6 +1656,7 @@ nsDOMWorker::CompileGlobalObject(JSContext* aCx)
   if (NS_FAILED(rv)) {
     mGlobal = NULL;
     mInnerScope = nsnull;
+    mScopeWN = nsnull;
     return PR_FALSE;
   }
 
@@ -1362,8 +1675,11 @@ nsDOMWorker::SetPool(nsDOMWorkerPool* aPool)
 already_AddRefed<nsIXPConnectWrappedNative>
 nsDOMWorker::GetWrappedNative()
 {
-  nsCOMPtr<nsIXPConnectWrappedNative> wrappedNative = mWrappedNative;
-  NS_ASSERTION(wrappedNative, "Null wrapped native!");
+  nsCOMPtr<nsIXPConnectWrappedNative> wrappedNative;
+  {
+    nsAutoLock lock(mLock);
+    wrappedNative = mWrappedNative;
+  }
   return wrappedNative.forget();
 }
 
@@ -1380,6 +1696,11 @@ nsDOMWorker::AddFeature(nsDOMWorkerFeature* aFeature,
 
     nsAutoLock lock(mLock);
 
+    if (mStatus == eKilled) {
+      // No features may be added after we've been canceled. Sorry.
+      return NS_ERROR_FAILURE;
+    }
+
     nsDOMWorkerFeature** newFeature = mFeatures.AppendElement(aFeature);
     NS_ENSURE_TRUE(newFeature, NS_ERROR_OUT_OF_MEMORY);
 
@@ -1491,37 +1812,128 @@ nsDOMWorker::ResumeFeatures()
   }
 }
 
-void
-nsDOMWorker::CancelFeatures()
+nsresult
+nsDOMWorker::SetURI(nsIURI* aURI)
 {
-  NS_ASSERTION(IsCanceled(), "More items can still be added!");
+  NS_ASSERTION(aURI, "Don't hand me a null pointer!");
+  NS_ASSERTION(!mURI && !mLocation, "Called more than once?!");
+  NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
 
-  PRUint32 count, index;
+  mURI = aURI;
 
-  nsAutoTArray<nsRefPtr<nsDOMWorkerFeature>, 20> features;
+  nsCOMPtr<nsIURL> url(do_QueryInterface(aURI));
+  NS_ENSURE_TRUE(url, NS_ERROR_NO_INTERFACE);
+
+  mLocation = nsDOMWorkerLocation::NewLocation(url);
+  NS_ENSURE_TRUE(mLocation, NS_ERROR_FAILURE);
+
+  return NS_OK;
+}
+
+nsresult
+nsDOMWorker::FireCloseRunnable(PRIntervalTime aTimeoutInterval,
+                               PRBool aClearQueue,
+                               PRBool aFromFinalize)
+{
+  // Resume the worker (but not its features) if we're currently suspended. This
+  // should only ever happen if we are being called from Cancel (page falling
+  // out of bfcache or quitting) or Finalize, in which case all we really want
+  // to do is unblock the waiting thread.
+  PRBool wakeUp;
   {
     nsAutoLock lock(mLock);
+    NS_ASSERTION(mExpirationTime == 0,
+                 "Close runnable should not be scheduled already!");
 
-    count = mFeatures.Length();
-    for (index = 0; index < count; index++) {
-      nsDOMWorkerFeature*& feature = mFeatures[index];
-
-#ifdef DEBUG
-      nsRefPtr<nsDOMWorkerFeature>* newFeature =
-#endif
-      features.AppendElement(feature);
-      NS_ASSERTION(newFeature, "Out of memory!");
-
-      feature->FreeToDie(PR_TRUE);
+    if ((wakeUp = mSuspended)) {
+      NS_ASSERTION(mStatus == eCanceled ||
+                   (mStatus == eTerminated && aFromFinalize),
+                   "How can this happen otherwise?!");
+      mSuspended = PR_FALSE;
     }
-
-    mFeatures.Clear();
   }
 
-  count = features.Length();
-  for (index = 0; index < count; index++) {
-    features[index]->Cancel();
+  if (wakeUp) {
+    nsAutoMonitor mon(mPool->Monitor());
+    mon.NotifyAll();
   }
+
+  nsRefPtr<nsDOMWorkerEvent> event = new nsDOMWorkerEvent();
+  NS_ENSURE_TRUE(event, NS_ERROR_OUT_OF_MEMORY);
+
+  nsresult rv =
+    event->InitEvent(NS_LITERAL_STRING("close"), PR_FALSE, PR_FALSE);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsRefPtr<nsDOMFireEventRunnable> runnable =
+    new nsDOMFireEventRunnable(this, event, PR_TRUE);
+  NS_ENSURE_TRUE(runnable, NS_ERROR_OUT_OF_MEMORY);
+
+  // Our worker has been collected and we want to keep the inner scope alive,
+  // so pass that along in the runnable.
+  if (aFromFinalize) {
+    NS_ASSERTION(mScopeWN, "This shouldn't be null!");
+    runnable->ReplaceWrappedNative(mScopeWN);
+  }
+
+  rv = nsDOMThreadService::get()->Dispatch(this, runnable, aTimeoutInterval,
+                                           aClearQueue);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  return NS_OK;
+}
+
+nsresult
+nsDOMWorker::Close()
+{
+  {
+    nsAutoLock lock(mLock);
+    NS_ASSERTION(mStatus != eKilled, "This should be impossible!");
+    if (mStatus != eRunning) {
+      return NS_OK;
+    }
+    mStatus = eClosed;
+  }
+
+  nsresult rv = FireCloseRunnable(PR_INTERVAL_NO_TIMEOUT, PR_FALSE, PR_FALSE);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  return NS_OK;
+}
+
+nsresult
+nsDOMWorker::TerminateInternal(PRBool aFromFinalize)
+{
+  {
+    nsAutoLock lock(mLock);
+#ifdef DEBUG
+    if (!aFromFinalize) {
+      NS_ASSERTION(mStatus != eCanceled, "Shouldn't be able to get here!");
+    }
+#endif
+
+    if (mStatus == eRunning) {
+      // This is the beginning of the close process, fire an event and prevent
+      // any other close events from being generated.
+      mStatus = eTerminated;
+    }
+    else {
+      if (mStatus == eClosed) {
+        // The worker was previously closed which means that an expiration time
+        // might not be set. Setting the status to eTerminated will force the
+        // worker to jump to its close handler.
+        mStatus = eTerminated;
+      }
+      // No need to fire another close handler, it has already been done.
+      return NS_OK;
+    }
+  }
+
+  nsresult rv = FireCloseRunnable(PR_INTERVAL_NO_TIMEOUT, PR_TRUE,
+                                  aFromFinalize);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  return NS_OK;
 }
 
 already_AddRefed<nsDOMWorker>
@@ -1531,23 +1943,100 @@ nsDOMWorker::GetParent()
   return parent.forget();
 }
 
+void
+nsDOMWorker::SetExpirationTime(PRIntervalTime aExpirationTime)
+{
+  {
+    nsAutoLock lock(mLock);
+
+    NS_ASSERTION(mStatus != eRunning && mStatus != eKilled, "Bad status!");
+    NS_ASSERTION(!mExpirationTime || mExpirationTime == PR_INTERVAL_NO_TIMEOUT,
+                 "Overwriting a timeout that was previously set!");
+
+    mExpirationTime = aExpirationTime;
+  }
+}
+
+#ifdef DEBUG
+PRIntervalTime
+nsDOMWorker::GetExpirationTime()
+{
+  nsAutoLock lock(mLock);
+  return mExpirationTime;
+}
+#endif
+
+PRBool
+nsDOMWorker::QueueSuspendedRunnable(nsIRunnable* aRunnable)
+{
+  NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
+  return mQueuedRunnables.AppendElement(aRunnable) ? PR_TRUE : PR_FALSE;
+}
+
+NS_IMETHODIMP
+nsDOMWorker::AddEventListener(const nsAString& aType,
+                              nsIDOMEventListener* aListener,
+                              PRBool aUseCapture)
+{
+  NS_ASSERTION(mWrappedNative, "Called after Finalize!");
+  if (IsCanceled()) {
+    return NS_OK;
+  }
+
+  return nsDOMWorkerMessageHandler::AddEventListener(aType, aListener,
+                                                     aUseCapture);
+}
+
+NS_IMETHODIMP
+nsDOMWorker::RemoveEventListener(const nsAString& aType,
+                                 nsIDOMEventListener* aListener,
+                                 PRBool aUseCapture)
+{
+  if (IsCanceled()) {
+    return NS_OK;
+  }
+
+  return nsDOMWorkerMessageHandler::RemoveEventListener(aType, aListener,
+                                                        aUseCapture);
+}
+
+NS_IMETHODIMP
+nsDOMWorker::DispatchEvent(nsIDOMEvent* aEvent,
+                           PRBool* _retval)
+{
+  {
+    nsAutoLock lock(mLock);
+    if (IsCanceledNoLock()) {
+      return NS_OK;
+    }
+    if (mStatus == eTerminated) {
+      nsCOMPtr<nsIWorkerMessageEvent> messageEvent(do_QueryInterface(aEvent));
+      if (messageEvent) {
+        // This is a message event targeted to a terminated worker. Ignore it.
+        return NS_OK;
+      }
+    }
+  }
+
+  return nsDOMWorkerMessageHandler::DispatchEvent(aEvent, _retval);
+}
+
 /**
  * See nsIWorker
  */
 NS_IMETHODIMP
 nsDOMWorker::PostMessage(/* JSObject aMessage */)
 {
-  if (mTerminated) {
-    return NS_OK;
+  {
+    nsAutoLock lock(mLock);
+    // There's no reason to dispatch this message after the close handler has
+    // been triggered since it will never be allowed to run.
+    if (mStatus != eRunning) {
+      return NS_OK;
+    }
   }
 
-  nsString message;
-  PRBool isJSON, isPrimitive;
-
-  nsresult rv = GetStringForArgument(message, &isJSON, &isPrimitive);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  return PostMessageInternal(message, isJSON, isPrimitive, PR_TRUE);
+  return PostMessageInternal(PR_TRUE);
 }
 
 /**
@@ -1558,8 +2047,13 @@ nsDOMWorker::GetOnerror(nsIDOMEventListener** aOnerror)
 {
   NS_ENSURE_ARG_POINTER(aOnerror);
 
+  if (IsCanceled()) {
+    *aOnerror = nsnull;
+    return NS_OK;
+  }
+
   nsCOMPtr<nsIDOMEventListener> listener =
-    mOuterHandler->GetOnXListener(NS_LITERAL_STRING("error"));
+    GetOnXListener(NS_LITERAL_STRING("error"));
 
   listener.forget(aOnerror);
   return NS_OK;
@@ -1571,7 +2065,12 @@ nsDOMWorker::GetOnerror(nsIDOMEventListener** aOnerror)
 NS_IMETHODIMP
 nsDOMWorker::SetOnerror(nsIDOMEventListener* aOnerror)
 {
-  return mOuterHandler->SetOnXListener(NS_LITERAL_STRING("error"), aOnerror);
+  NS_ASSERTION(mWrappedNative, "Called after Finalize!");
+  if (IsCanceled()) {
+    return NS_OK;
+  }
+
+  return SetOnXListener(NS_LITERAL_STRING("error"), aOnerror);
 }
 
 /**
@@ -1582,8 +2081,13 @@ nsDOMWorker::GetOnmessage(nsIDOMEventListener** aOnmessage)
 {
   NS_ENSURE_ARG_POINTER(aOnmessage);
 
+  if (IsCanceled()) {
+    *aOnmessage = nsnull;
+    return NS_OK;
+  }
+
   nsCOMPtr<nsIDOMEventListener> listener =
-    mOuterHandler->GetOnXListener(NS_LITERAL_STRING("message"));
+    GetOnXListener(NS_LITERAL_STRING("message"));
 
   listener.forget(aOnmessage);
   return NS_OK;
@@ -1595,21 +2099,24 @@ nsDOMWorker::GetOnmessage(nsIDOMEventListener** aOnmessage)
 NS_IMETHODIMP
 nsDOMWorker::SetOnmessage(nsIDOMEventListener* aOnmessage)
 {
-  return mOuterHandler->SetOnXListener(NS_LITERAL_STRING("message"),
-                                       aOnmessage);
+  NS_ASSERTION(mWrappedNative, "Called after Finalize!");
+  if (IsCanceled()) {
+    return NS_OK;
+  }
+
+  return SetOnXListener(NS_LITERAL_STRING("message"), aOnmessage);
 }
 
 NS_IMETHODIMP
 nsDOMWorker::Terminate()
 {
-  if (mCanceled || mTerminated) {
-    return NS_OK;
-  }
+  return TerminateInternal(PR_FALSE);
+}
 
-  mTerminated = PR_TRUE;
-
-  nsCOMPtr<nsIRunnable> runnable = new nsCancelDOMWorkerRunnable(this);
-  NS_ENSURE_TRUE(runnable, NS_ERROR_OUT_OF_MEMORY);
-
-  return NS_DispatchToMainThread(runnable, NS_DISPATCH_NORMAL);
+NS_IMETHODIMP
+nsDOMWorker::Notify(nsITimer* aTimer)
+{
+  NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
+  Kill();
+  return NS_OK;
 }
