@@ -19,7 +19,7 @@
  * Portions created by the Initial Developer are Copyright (C) 2008
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Jesse Ruderman
+ * Contributor(s): nanto_vi (TOYAMA Nao)
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -35,12 +35,12 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-var gTestfile = 'regress-452498-075.js';
+var gTestfile = 'regress-433279-01.js';
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 452498;
-var summary = 'TM: upvar2 regression tests';
-var actual = '';
-var expect = '';
+var BUGNUMBER = 433279;
+var summary = 'Do not assert: pn != tc->parseContext->nodeList';
+var actual = 'No Crash';
+var expect = 'No Crash';
 
 
 //-----------------------------------------------------------------------------
@@ -52,14 +52,10 @@ function test()
   enterFunc ('test');
   printBugNumber(BUGNUMBER);
   printStatus (summary);
+ 
+  var { sin, PI } = Math; sin(PI / 2);
 
-// ------- Comment #75 From Jesse Ruderman
-
-  (function p(){ p = 3; });
-  (function p(){ p = 3; return p; })()
-// Assertion failure: regs.sp == StackBase(fp), at ../jsinterp.cpp:2980
-
-    reportCompare(expect, actual, summary);
+  reportCompare(expect, actual, summary);
 
   exitFunc ('test');
 }
