@@ -35,26 +35,25 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-var gTestfile = 'regress-469405-01.js';
+var gTestfile = 'regress-452498-162.js';
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 469405;
-var summary = 'Do not assert: !JSVAL_IS_PRIMITIVE(regs.sp[-2])';
+var BUGNUMBER = 452498;
+var summary = 'TM: upvar2 regression tests';
 var actual = '';
 var expect = '';
 
+//-------  Comment #162  From  Gary Kwong
+
 printBugNumber(BUGNUMBER);
 printStatus (summary);
 
-try
-{ 
-  (function() {
-    var a, b;
-    for each (a in [{}, {__iterator__: function(){}}]) for (b in a) { }
-  })();
-}
-catch(ex)
-{
-  print('caught ' + ex);
-}
+// Assertion failure: !OBJ_GET_CLASS(cx, proto)->getObjectOps, at ../jsobj.cpp:2030
+
+jit(true);
+__defineGetter__("x3", Function);
+undefined = x3;
+undefined.prototype = [];
+for (var z = 0; z < 4; ++z) { new undefined() }
+jit(false);
 
 reportCompare(expect, actual, summary);
