@@ -48,28 +48,45 @@ MAKEFILES_db="
 "
 
 MAKEFILES_dom="
+  ipc/Makefile
+  ipc/chromium/Makefile
+  ipc/glue/Makefile
+  ipc/ipdl/Makefile
+  ipc/testshell/Makefile
   dom/Makefile
-  dom/public/Makefile
-  dom/public/base/Makefile
   dom/public/coreEvents/Makefile
-  dom/public/idl/Makefile
-  dom/public/idl/base/Makefile
-  dom/public/idl/canvas/Makefile
-  dom/public/idl/core/Makefile
-  dom/public/idl/css/Makefile
-  dom/public/idl/events/Makefile
-  dom/public/idl/html/Makefile
-  dom/public/idl/range/Makefile
-  dom/public/idl/stylesheets/Makefile
-  dom/public/idl/views/Makefile
-  dom/public/idl/xbl/Makefile
-  dom/public/idl/xpath/Makefile
-  dom/public/idl/xul/Makefile
+  dom/interfaces/base/Makefile
+  dom/interfaces/canvas/Makefile
+  dom/interfaces/core/Makefile
+  dom/interfaces/css/Makefile
+  dom/interfaces/events/Makefile
+  dom/interfaces/geolocation/Makefile
+  dom/interfaces/html/Makefile
+  dom/interfaces/json/Makefile
+  dom/interfaces/load-save/Makefile
+  dom/interfaces/offline/Makefile
+  dom/interfaces/range/Makefile
+  dom/interfaces/sidebar/Makefile
+  dom/interfaces/storage/Makefile
+  dom/interfaces/stylesheets/Makefile
+  dom/interfaces/threads/Makefile
+  dom/interfaces/traversal/Makefile
+  dom/interfaces/views/Makefile
+  dom/interfaces/xbl/Makefile
+  dom/interfaces/xpath/Makefile
+  dom/interfaces/xul/Makefile
+  dom/base/Makefile
   dom/src/Makefile
-  dom/src/base/Makefile
   dom/src/events/Makefile
   dom/src/jsurl/Makefile
+  dom/src/geolocation/Makefile
+  dom/src/json/Makefile
+  dom/src/offline/Makefile
+  dom/src/storage/Makefile
+  dom/src/threads/Makefile
   dom/locales/Makefile
+  dom/plugins/Makefile
+  dom/ipc/Makefile
 "
 
 MAKEFILES_editor="
@@ -95,9 +112,12 @@ MAKEFILES_editor="
   editor/libeditor/text/Makefile
 "
 
-MAKEFILES_expat="
+MAKEFILES_xmlparser="
   parser/expat/Makefile
   parser/expat/lib/Makefile
+  parser/xml/Makefile
+  parser/xml/public/Makefile
+  parser/xml/src/Makefile
 "
 
 MAKEFILES_gfx="
@@ -108,6 +128,9 @@ MAKEFILES_gfx="
   gfx/src/psshared/Makefile
   gfx/src/thebes/Makefile
   gfx/tests/Makefile
+  gfx/thebes/Makefile
+  gfx/thebes/public/Makefile
+  gfx/thebes/src/Makefile
   gfx/qcms/Makefile
 "
 
@@ -124,11 +147,13 @@ MAKEFILES_htmlparser="
 
 MAKEFILES_intl="
   intl/Makefile
+  intl/build/Makefile
   intl/chardet/Makefile
   intl/chardet/public/Makefile
   intl/chardet/src/Makefile
   intl/uconv/Makefile
   intl/uconv/idl/Makefile
+  intl/uconv/util/Makefile
   intl/uconv/public/Makefile
   intl/uconv/src/Makefile
   intl/uconv/tests/Makefile
@@ -144,11 +169,13 @@ MAKEFILES_intl="
   intl/locale/public/Makefile
   intl/locale/idl/Makefile
   intl/locale/src/Makefile
+  intl/locale/src/mac/Makefile
   intl/locale/src/unix/Makefile
   intl/locale/src/os2/Makefile
   intl/locale/src/windows/Makefile
   intl/locale/tests/Makefile
   intl/lwbrk/Makefile
+  intl/lwbrk/idl/Makefile
   intl/lwbrk/src/Makefile
   intl/lwbrk/public/Makefile
   intl/lwbrk/tests/Makefile
@@ -191,6 +218,10 @@ MAKEFILES_jsdebugger="
   js/jsd/idl/Makefile
 "
 
+MAKEFILES_jsctypes="
+  js/ctypes/Makefile
+"
+
 MAKEFILES_content="
   content/Makefile
   content/base/Makefile
@@ -216,6 +247,7 @@ MAKEFILES_content="
   content/xml/content/src/Makefile
   content/xml/document/Makefile
   content/xml/document/public/Makefile
+  content/xml/document/resources/Makefile
   content/xml/document/src/Makefile
   content/xul/Makefile
   content/xul/content/Makefile
@@ -223,12 +255,17 @@ MAKEFILES_content="
   content/xul/document/Makefile
   content/xul/document/public/Makefile
   content/xul/document/src/Makefile
+  content/xul/templates/Makefile
   content/xul/templates/public/Makefile
   content/xul/templates/src/Makefile
+  content/xul/templates/tests/Makefile
+  content/xul/templates/tests/chrome/Makefile
   content/xbl/Makefile
   content/xbl/public/Makefile
   content/xbl/src/Makefile
   content/xbl/builtin/Makefile
+  content/xbl/builtin/gtk2/Makefile
+  content/xbl/builtin/mac/Makefile
   content/xslt/Makefile
   content/xslt/public/Makefile
   content/xslt/src/Makefile
@@ -245,11 +282,17 @@ MAKEFILES_layout="
   layout/base/tests/Makefile
   layout/build/Makefile
   layout/forms/Makefile
+  layout/generic/Makefile
   layout/html/tests/Makefile
-  layout/style/Makefile
+  layout/inspector/public/Makefile
+  layout/inspector/src/Makefile
   layout/printing/Makefile
+  layout/style/Makefile
+  layout/style/xbl-marquee/Makefile
+  layout/tables/Makefile
   layout/xul/base/public/Makefile
   layout/xul/base/src/Makefile
+  layout/xul/base/src/grid/Makefile
   layout/xul/base/src/tree/src/Makefile
   layout/xul/base/src/tree/public/Makefile
 "
@@ -260,7 +303,6 @@ MAKEFILES_libimg="
 
 MAKEFILES_libjar="
   modules/libjar/Makefile
-  modules/libjar/standalone/Makefile
   modules/libjar/test/Makefile
 "
 
@@ -345,7 +387,9 @@ MAKEFILES_plugin="
   modules/plugin/Makefile
   modules/plugin/base/src/Makefile
   modules/plugin/base/public/Makefile
+  modules/plugin/default/mac/Makefile
   modules/plugin/default/unix/Makefile
+  modules/plugin/default/windows/Makefile
   modules/plugin/sdk/Makefile
   modules/plugin/sdk/samples/Makefile
   modules/plugin/sdk/samples/common/Makefile
@@ -353,6 +397,10 @@ MAKEFILES_plugin="
   modules/plugin/sdk/samples/winless/windows/Makefile
 "
 
+MAKEFILES_freetype2="
+  modules/freetype2/Makefile
+"
+
 MAKEFILES_netwerk="
   netwerk/Makefile
   netwerk/base/Makefile
@@ -391,6 +439,9 @@ MAKEFILES_netwerk="
   netwerk/protocol/res/Makefile
   netwerk/protocol/res/public/Makefile
   netwerk/protocol/res/src/Makefile
+  netwerk/protocol/viewsource/Makefile
+  netwerk/protocol/viewsource/public/Makefile
+  netwerk/protocol/viewsource/src/Makefile
   netwerk/mime/Makefile
   netwerk/mime/public/Makefile
   netwerk/mime/src/Makefile
@@ -402,9 +453,9 @@ MAKEFILES_netwerk="
   netwerk/streamconv/src/Makefile
   netwerk/streamconv/test/Makefile
   netwerk/test/Makefile
-  netwerk/testserver/Makefile
   netwerk/locales/Makefile
   netwerk/system/Makefile
+  netwerk/system/mac/Makefile
   netwerk/system/win32/Makefile
 "
 
@@ -413,6 +464,7 @@ MAKEFILES_uriloader="
   uriloader/base/Makefile
   uriloader/exthandler/Makefile
   uriloader/exthandler/tests/Makefile
+  uriloader/prefetch/Makefile
 "
 
 MAKEFILES_profile="
@@ -421,6 +473,7 @@ MAKEFILES_profile="
   profile/dirserviceprovider/Makefile
   profile/dirserviceprovider/public/Makefile
   profile/dirserviceprovider/src/Makefile
+  profile/dirserviceprovider/standalone/Makefile
 "
 
 MAKEFILES_rdf="
@@ -454,6 +507,8 @@ MAKEFILES_caps="
   caps/idl/Makefile
   caps/include/Makefile
   caps/src/Makefile
+  caps/tests/Makefile
+  caps/tests/mochitest/Makefile
 "
 
 MAKEFILES_chrome="
@@ -471,6 +526,8 @@ MAKEFILES_view="
 MAKEFILES_docshell="
   docshell/Makefile
   docshell/base/Makefile
+  docshell/resources/Makefile
+  docshell/resources/content/Makefile
   docshell/shistory/Makefile
   docshell/shistory/public/Makefile
   docshell/shistory/src/Makefile
@@ -488,13 +545,13 @@ MAKEFILES_widget="
   widget/src/Makefile
   widget/src/beos/Makefile
   widget/src/build/Makefile
+  widget/src/gtk2/Makefile
   widget/src/gtkxtbin/Makefile
   widget/src/photon/Makefile
   widget/src/cocoa/Makefile
   widget/src/os2/Makefile
   widget/src/windows/Makefile
   widget/src/xpwidgets/Makefile
-  widget/src/support/Makefile
 "
 
 MAKEFILES_xpcom="
@@ -583,9 +640,9 @@ MAKEFILES_xpfe="
   xpfe/appshell/Makefile
   xpfe/appshell/src/Makefile
   xpfe/appshell/public/Makefile
-  xpfe/bootstrap/appleevents/Makefile
   extensions/spellcheck/Makefile
   extensions/spellcheck/hunspell/Makefile
+  extensions/spellcheck/hunspell/src/Makefile
   extensions/spellcheck/idl/Makefile
   extensions/spellcheck/locales/Makefile
   extensions/spellcheck/src/Makefile
@@ -594,10 +651,13 @@ MAKEFILES_xpfe="
 MAKEFILES_embedding="
   embedding/Makefile
   embedding/base/Makefile
+  embedding/base/standalone/Makefile
   embedding/browser/Makefile
   embedding/browser/activex/src/Makefile
+  embedding/browser/activex/src/common/Makefile
   embedding/browser/activex/src/control/Makefile
   embedding/browser/activex/src/control_kicker/Makefile
+  embedding/browser/activex/src/plugin/Makefile
   embedding/browser/build/Makefile
   embedding/browser/webBrowser/Makefile
   embedding/browser/gtk/Makefile
@@ -607,7 +667,23 @@ MAKEFILES_embedding="
   embedding/browser/photon/src/Makefile
   embedding/browser/photon/tests/Makefile
   embedding/components/Makefile
+  embedding/components/appstartup/Makefile
+  embedding/components/appstartup/src/Makefile
   embedding/components/build/Makefile
+  embedding/components/commandhandler/Makefile
+  embedding/components/commandhandler/public/Makefile
+  embedding/components/commandhandler/src/Makefile
+  embedding/components/find/Makefile
+  embedding/components/find/public/Makefile
+  embedding/components/find/src/Makefile
+  embedding/components/printingui/Makefile
+  embedding/components/printingui/src/Makefile
+  embedding/components/printingui/src/mac/Makefile
+  embedding/components/printingui/src/unixshared/Makefile
+  embedding/components/printingui/src/win/Makefile
+  embedding/components/webbrowserpersist/Makefile
+  embedding/components/webbrowserpersist/public/Makefile
+  embedding/components/webbrowserpersist/src/Makefile
   embedding/components/windowwatcher/Makefile
   embedding/components/windowwatcher/public/Makefile
   embedding/components/windowwatcher/src/Makefile
@@ -625,19 +701,30 @@ MAKEFILES_xulapp="
   toolkit/crashreporter/Makefile
   toolkit/crashreporter/client/Makefile
   toolkit/crashreporter/google-breakpad/src/client/Makefile
+  toolkit/crashreporter/google-breakpad/src/client/linux/handler/Makefile
+  toolkit/crashreporter/google-breakpad/src/client/linux/minidump_writer/Makefile
   toolkit/crashreporter/google-breakpad/src/client/mac/handler/Makefile
-  toolkit/crashreporter/google-breakpad/src/client/windows/Makefile
+  toolkit/crashreporter/google-breakpad/src/client/solaris/handler/Makefile
+  toolkit/crashreporter/google-breakpad/src/client/windows/crash_generation/Makefile
   toolkit/crashreporter/google-breakpad/src/client/windows/handler/Makefile
   toolkit/crashreporter/google-breakpad/src/client/windows/sender/Makefile
   toolkit/crashreporter/google-breakpad/src/common/Makefile
+  toolkit/crashreporter/google-breakpad/src/common/linux/Makefile
   toolkit/crashreporter/google-breakpad/src/common/mac/Makefile
+  toolkit/crashreporter/google-breakpad/src/common/mac/dwarf/Makefile
+  toolkit/crashreporter/google-breakpad/src/common/solaris/Makefile
   toolkit/crashreporter/google-breakpad/src/common/windows/Makefile
+  toolkit/crashreporter/google-breakpad/src/tools/linux/dump_syms/Makefile
   toolkit/crashreporter/google-breakpad/src/tools/mac/dump_syms/Makefile
+  toolkit/crashreporter/google-breakpad/src/tools/solaris/dump_syms/Makefile
   toolkit/content/Makefile
   toolkit/obsolete/Makefile
   toolkit/components/alerts/Makefile
   toolkit/components/alerts/public/Makefile
   toolkit/components/alerts/src/Makefile
+  toolkit/components/alerts/src/mac/Makefile
+  toolkit/components/alerts/src/mac/growl/Makefile
+  toolkit/components/apppicker/Makefile
   toolkit/components/autocomplete/Makefile
   toolkit/components/autocomplete/public/Makefile
   toolkit/components/autocomplete/src/Makefile
@@ -647,17 +734,26 @@ MAKEFILES_xulapp="
   toolkit/components/commandlines/public/Makefile
   toolkit/components/commandlines/src/Makefile
   toolkit/components/console/Makefile
+  toolkit/components/contentprefs/Makefile
+  toolkit/components/contentprefs/public/Makefile
+  toolkit/components/contentprefs/src/Makefile
   toolkit/components/cookie/Makefile
   toolkit/components/downloads/public/Makefile
   toolkit/components/downloads/Makefile
   toolkit/components/downloads/src/Makefile
+  toolkit/components/exthelper/Makefile
+  toolkit/components/feeds/Makefile
+  toolkit/components/feeds/public/Makefile
+  toolkit/components/feeds/src/Makefile
   toolkit/components/filepicker/Makefile
   toolkit/components/filepicker/public/Makefile
   toolkit/components/filepicker/src/Makefile
-  toolkit/system/gnome/Makefile
-  toolkit/system/unixproxy/Makefile
-  toolkit/system/osxproxy/Makefile
   toolkit/components/help/Makefile
+  toolkit/components/microformats/Makefile
+  toolkit/components/microformats/src/Makefile
+  toolkit/components/parentalcontrols/Makefile
+  toolkit/components/parentalcontrols/public/Makefile
+  toolkit/components/parentalcontrols/src/Makefile
   toolkit/components/passwordmgr/Makefile
   toolkit/components/passwordmgr/public/Makefile
   toolkit/components/passwordmgr/src/Makefile
@@ -670,12 +766,20 @@ MAKEFILES_xulapp="
   toolkit/components/satchel/Makefile
   toolkit/components/satchel/public/Makefile
   toolkit/components/satchel/src/Makefile
+  toolkit/components/search/Makefile
+  toolkit/spatial-navigation/Makefile
   toolkit/components/startup/Makefile
   toolkit/components/startup/public/Makefile
   toolkit/components/startup/src/Makefile
   toolkit/components/typeaheadfind/Makefile
   toolkit/components/typeaheadfind/public/Makefile
   toolkit/components/typeaheadfind/src/Makefile
+  toolkit/components/url-classifier/Makefile
+  toolkit/components/url-classifier/public/Makefile
+  toolkit/components/url-classifier/src/Makefile
+  toolkit/components/urlformatter/Makefile
+  toolkit/components/urlformatter/public/Makefile
+  toolkit/components/urlformatter/src/Makefile
   toolkit/components/viewconfig/Makefile
   toolkit/components/viewsource/Makefile
   toolkit/locales/Makefile
@@ -685,17 +789,27 @@ MAKEFILES_xulapp="
   toolkit/mozapps/extensions/Makefile
   toolkit/mozapps/extensions/public/Makefile
   toolkit/mozapps/extensions/src/Makefile
+  toolkit/mozapps/handling/Makefile
+  toolkit/mozapps/handling/src/Makefile
+  toolkit/mozapps/plugins/Makefile
   toolkit/mozapps/update/Makefile
   toolkit/mozapps/update/public/Makefile
   toolkit/mozapps/update/src/Makefile
+  toolkit/mozapps/update/src/updater/Makefile
   toolkit/mozapps/xpinstall/Makefile
   toolkit/profile/Makefile
   toolkit/profile/public/Makefile
   toolkit/profile/skin/Makefile
   toolkit/profile/src/Makefile
+  toolkit/system/dbus/Makefile
+  toolkit/system/gnome/Makefile
+  toolkit/system/unixproxy/Makefile
+  toolkit/system/osxproxy/Makefile
+  toolkit/system/windowsproxy/Makefile
   toolkit/themes/Makefile
   toolkit/themes/gnomestripe/global/Makefile
   toolkit/themes/gnomestripe/Makefile
+  toolkit/themes/gnomestripe/mozapps/Makefile
   toolkit/themes/pmstripe/global/Makefile
   toolkit/themes/pmstripe/Makefile
   toolkit/themes/pinstripe/Makefile
@@ -720,10 +834,10 @@ MAKEFILES_libpr0n="
   modules/libpr0n/decoders/jpeg/Makefile
   modules/libpr0n/decoders/bmp/Makefile
   modules/libpr0n/decoders/icon/Makefile
+  modules/libpr0n/decoders/icon/mac/Makefile
   modules/libpr0n/decoders/icon/win/Makefile
   modules/libpr0n/decoders/icon/gtk/Makefile
   modules/libpr0n/decoders/icon/beos/Makefile
-  modules/libpr0n/decoders/xbm/Makefile
   modules/libpr0n/encoders/Makefile
   modules/libpr0n/encoders/png/Makefile
   modules/libpr0n/encoders/jpeg/Makefile
@@ -732,10 +846,12 @@ MAKEFILES_libpr0n="
 MAKEFILES_accessible="
   accessible/Makefile
   accessible/public/Makefile
+  accessible/public/ia2/Makefile
   accessible/public/msaa/Makefile
   accessible/src/Makefile
   accessible/src/base/Makefile
   accessible/src/html/Makefile
+  accessible/src/xforms/Makefile
   accessible/src/xul/Makefile
   accessible/src/msaa/Makefile
   accessible/src/atk/Makefile
@@ -753,17 +869,31 @@ MAKEFILES_libmar="
   modules/libmar/tool/Makefile
 "
 
+MAKEFILES_lib7z="
+  modules/lib7z/Makefile
+"
+
+MAKEFILES_extensions="
+  extensions/cookie/Makefile
+  extensions/permissions/Makefile
+  extensions/pref/Makefile
+  extensions/pref/autoconfig/Makefile
+  extensions/pref/autoconfig/public/Makefile
+  extensions/pref/autoconfig/src/Makefile
+"
+
 add_makefiles "
   $MAKEFILES_db
   $MAKEFILES_dom
   $MAKEFILES_editor
-  $MAKEFILES_expat
+  $MAKEFILES_xmlparser
   $MAKEFILES_gfx
   $MAKEFILES_htmlparser
   $MAKEFILES_intl
   $MAKEFILES_liveconnect
   $MAKEFILES_xpconnect
   $MAKEFILES_jsdebugger
+  $MAKEFILES_jsctypes
   $MAKEFILES_content
   $MAKEFILES_layout
   $MAKEFILES_libimg
@@ -795,12 +925,162 @@ add_makefiles "
   $MAKEFILES_accessible
   $MAKEFILES_zlib
   $MAKEFILES_libmar
+  $MAKEFILES_lib7z
+  $MAKEFILES_extensions
 "
 
 #
 # Conditional makefiles
 #
 
+if [ "$ENABLE_TESTS" ]; then
+  add_makefiles "
+    chrome/test/Makefile
+    content/canvas/test/Makefile
+    content/events/test/Makefile
+    content/html/content/test/Makefile
+    content/html/document/test/Makefile
+    content/smil/test/Makefile
+    content/svg/content/test/Makefile
+    content/test/Makefile
+    content/xbl/test/Makefile
+    content/xml/document/test/Makefile
+    content/xslt/tests/buster/Makefile
+    content/xslt/tests/mochitest/Makefile
+    content/xtf/test/Makefile
+    content/xul/content/test/Makefile
+    content/xul/document/test/Makefile
+    docshell/test/Makefile
+    docshell/test/browser/Makefile
+    docshell/test/chrome/Makefile
+    docshell/test/navigation/Makefile
+    dom/src/json/test/Makefile
+    dom/src/jsurl/test/Makefile
+    dom/src/threads/test/Makefile
+    dom/tests/Makefile
+    dom/tests/mochitest/Makefile
+    dom/tests/mochitest/ajax/Makefile
+    dom/tests/mochitest/ajax/jquery/Makefile
+    dom/tests/mochitest/ajax/jquery/dist/Makefile
+    dom/tests/mochitest/ajax/jquery/test/Makefile
+    dom/tests/mochitest/ajax/jquery/test/data/Makefile
+    dom/tests/mochitest/ajax/jquery/test/data/offset/Makefile
+    dom/tests/mochitest/ajax/jquery/test/unit/Makefile
+    dom/tests/mochitest/ajax/lib/Makefile
+    dom/tests/mochitest/ajax/mochikit/Makefile
+    dom/tests/mochitest/ajax/mochikit/MochiKit/Makefile
+    dom/tests/mochitest/ajax/mochikit/tests/Makefile
+    dom/tests/mochitest/ajax/mochikit/tests/SimpleTest/Makefile
+    dom/tests/mochitest/ajax/offline/Makefile
+    dom/tests/mochitest/ajax/offline/namespace1/Makefile
+    dom/tests/mochitest/ajax/offline/namespace1/sub/Makefile
+    dom/tests/mochitest/ajax/offline/namespace1/sub2/Makefile
+    dom/tests/mochitest/ajax/offline/namespace2/Makefile
+    dom/tests/mochitest/ajax/prototype/Makefile
+    dom/tests/mochitest/ajax/prototype/dist/Makefile
+    dom/tests/mochitest/ajax/prototype/test/Makefile
+    dom/tests/mochitest/ajax/prototype/test/functional/Makefile
+    dom/tests/mochitest/ajax/prototype/test/lib/Makefile
+    dom/tests/mochitest/ajax/prototype/test/unit/Makefile
+    dom/tests/mochitest/ajax/prototype/test/unit/fixtures/Makefile
+    dom/tests/mochitest/ajax/prototype/test/unit/tmp/Makefile
+    dom/tests/mochitest/ajax/scriptaculous/Makefile
+    dom/tests/mochitest/ajax/scriptaculous/lib/Makefile
+    dom/tests/mochitest/ajax/scriptaculous/src/Makefile
+    dom/tests/mochitest/ajax/scriptaculous/test/Makefile
+    dom/tests/mochitest/ajax/scriptaculous/test/unit/Makefile
+    dom/tests/mochitest/bugs/Makefile
+    dom/tests/mochitest/chrome/Makefile
+    dom/tests/mochitest/dom-level0/Makefile
+    dom/tests/mochitest/dom-level1-core/Makefile
+    dom/tests/mochitest/dom-level1-core/files/Makefile
+    dom/tests/mochitest/dom-level2-core/Makefile
+    dom/tests/mochitest/dom-level2-core/files/Makefile
+    dom/tests/mochitest/dom-level2-html/Makefile
+    dom/tests/mochitest/dom-level2-html/files/Makefile
+    dom/tests/mochitest/general/Makefile
+    dom/tests/mochitest/geolocation/Makefile
+    dom/tests/mochitest/localstorage/Makefile
+    dom/tests/mochitest/sessionstorage/Makefile
+    dom/tests/mochitest/whatwg/Makefile
+    editor/libeditor/html/tests/Makefile
+    editor/libeditor/text/tests/Makefile
+    embedding/test/Makefile
+    extensions/cookie/test/Makefile
+    extensions/pref/Makefile
+    intl/locale/tests_multilocale/Makefile
+    js/src/xpconnect/tests/mochitest/Makefile
+    js/ctypes/tests/Makefile
+    layout/forms/test/Makefile
+    layout/generic/test/Makefile
+    layout/inspector/tests/Makefile
+    layout/reftests/fonts/Makefile
+    layout/reftests/fonts/mplus/Makefile
+    layout/style/test/Makefile
+    layout/tables/test/Makefile
+    layout/tools/pageloader/Makefile
+    layout/tools/reftest/Makefile
+    layout/xul/base/test/Makefile
+    layout/xul/test/Makefile
+    modules/libjar/test/chrome/Makefile
+    modules/libjar/test/mochitest/Makefile
+    modules/libpr0n/test/Makefile
+    modules/libpr0n/test/mochitest/Makefile
+    modules/libpref/test/Makefile
+    modules/plugin/test/Makefile
+    modules/plugin/test/mochitest/Makefile
+    modules/plugin/test/testplugin/Makefile
+    netwerk/test/httpserver/Makefile
+    parser/htmlparser/tests/mochitest/Makefile
+    parser/xml/test/Makefile
+    rdf/tests/triplescat/Makefile
+    testing/mochitest/Makefile
+    testing/mochitest/MochiKit/Makefile
+    testing/mochitest/chrome/Makefile
+    testing/mochitest/ssltunnel/Makefile
+    testing/mochitest/static/Makefile
+    testing/mochitest/tests/Makefile
+    testing/mochitest/tests/SimpleTest/Makefile
+    testing/mochitest/tests/browser/Makefile
+    testing/xpcshell/Makefile
+    testing/xpcshell/example/Makefile
+    toolkit/components/alerts/test/Makefile
+    toolkit/components/autocomplete/tests/Makefile
+    toolkit/components/commandlines/test/Makefile
+    toolkit/components/contentprefs/tests/Makefile
+    toolkit/components/downloads/test/Makefile
+    toolkit/components/downloads/test/browser/Makefile
+    toolkit/components/microformats/tests/Makefile
+    toolkit/components/passwordmgr/test/browser/Makefile
+    toolkit/components/places/tests/Makefile
+    toolkit/components/places/tests/chrome/Makefile
+    toolkit/components/places/tests/mochitest/bug_411966/Makefile
+    toolkit/components/places/tests/mochitest/bug_461710/Makefile
+    toolkit/components/satchel/test/Makefile
+    toolkit/components/url-classifier/tests/Makefile
+    toolkit/components/url-classifier/tests/mochitest/Makefile
+    toolkit/components/urlformatter/tests/Makefile
+    toolkit/components/viewsource/test/Makefile
+    toolkit/content/tests/Makefile
+    toolkit/content/tests/browser/Makefile
+    toolkit/content/tests/chrome/Makefile
+    toolkit/content/tests/widgets/Makefile
+    toolkit/crashreporter/test/Makefile
+    toolkit/mozapps/downloads/tests/Makefile
+    toolkit/mozapps/downloads/tests/chrome/Makefile
+    toolkit/mozapps/extensions/test/Makefile
+    toolkit/mozapps/plugins/tests/Makefile
+    toolkit/mozapps/update/test/Makefile
+    toolkit/spatial-navigation/tests/Makefile
+    toolkit/xre/test/Makefile
+    uriloader/exthandler/tests/mochitest/Makefile
+    widget/tests/Makefile
+    xpcom/sample/program/Makefile
+    xpcom/tests/external/Makefile
+    xpinstall/tests/Makefile
+  "
+fi
+
 if [ "$MOZ_ZIPWRITER" ]; then
   add_makefiles "
     modules/libjar/zipwriter/Makefile
@@ -814,6 +1094,7 @@ if [ "$MOZ_STORAGE" ]; then
   add_makefiles "
     db/sqlite3/src/Makefile
     db/morkreader/Makefile
+    db/morkreader/external/Makefile
     storage/Makefile
     storage/public/Makefile
     storage/src/Makefile
@@ -969,9 +1250,11 @@ fi
 
 if [ "$MOZ_MATHML" ]; then
   add_makefiles "
+    content/mathml/Makefile
+    content/mathml/content/Makefile
+    content/mathml/content/src/Makefile
     intl/uconv/ucvmath/Makefile
-    layout/mathml/base/src/Makefile
-    layout/mathml/content/src/Makefile
+    layout/mathml/Makefile
   "
 fi
 
@@ -982,11 +1265,18 @@ if [ "$MOZ_SVG" ]; then
     content/svg/document/src/Makefile
     content/svg/content/Makefile
     content/svg/content/src/Makefile
-    dom/public/idl/svg/Makefile
+    dom/interfaces/svg/Makefile
     layout/svg/base/src/Makefile
   "
 fi
 
+if [ "$MOZ_SMIL" ]; then
+  add_makefiles "
+    content/smil/Makefile
+    dom/interfaces/smil/Makefile
+  "
+fi
+
 if [ "$MOZ_XTF" ]; then
   add_makefiles "
     content/xtf/Makefile
@@ -1051,10 +1341,7 @@ fi # MOZ_COMPONENTLIB
 if [ "$MOZ_MEDIA" ]; then
  add_makefiles "
    content/media/Makefile
-   content/media/video/Makefile
-   content/media/video/public/Makefile
-   content/media/video/src/Makefile
-   content/media/video/test/Makefile
+   content/media/test/Makefile
  "
 fi
 
@@ -1066,6 +1353,13 @@ if [ "$MOZ_OGG" ]; then
    $MAKEFILES_libogg
    $MAKEFILES_libfishsound
    $MAKEFILES_liboggplay
+   content/media/ogg/Makefile
+ "
+fi
+
+if [ "$MOZ_WAVE" ]; then
+ add_makefiles "
+   content/media/wave/Makefile
  "
 fi
 
@@ -1074,3 +1368,9 @@ if [ "$MOZ_SYDNEYAUDIO" ]; then
    $MAKEFILES_libsydneyaudio
  "
 fi
+
+if [ "$MOZ_TREE_FREETYPE" ]; then
+ add_makefiles "
+   $MAKEFILES_freetype2
+ "
+fi
